@@ -1,5 +1,7 @@
 from .argo_client import ArgoClient
 from .argo_options import ArgoOptions
+from .dsl import DAG, CronWorkflow, Task, Workflow, WorkflowTemplate
+from .dsl.condition import Condition
 from .exceptions.workflow_not_found_exception import WorkflowNotFoundException
 from .workflow_result import WorkflowResult
 from .workflow_status import WorkflowStatus
